#!/bin/sh

:<<'########################################################################'

  Usage: make_pdnsd.sh
    
This script downloads and constructs the pdnsd extension. See

------------------------------------------------------------------------

Change-log: 
  2011-10-09 First version.

------------------------------------------------------------------------

This script was written in 2011 by vitex from forum.tinycorelinux.net.

To the extent possible under law, the author(s) have dedicated all
copyright and related and neighboring rights to this software to the
public domain worldwide. This software is distributed without any
warranty.

See http://creativecommons.org/publicdomain/zero/1.0/ for a copy 
of the CC0 Public Domain Dedication under which this work is 
distributed.

########################################################################

# Specify the name of the extension.
EXTENSION=pdnsd

# Specify the base portion of the extension.
EXTENSION0=pdnsd

#-----------------------------------------------------------------------

# Install the extensions needed to build this extension.
tce-load -wi compiletc.tcz groff.tcz

#-----------------------------------------------------------------------

# Specify the temporary directory that will contain the build components.
TMP=/tmp/tcz

# Specify the directory in which the squashfs files will be assembled.
SQUASHFS=$TMP/squashfs 

# Specify the directory in which the submission package will be assembled.
PACKAGE=$TMP/package

#-----------------------------------------------------------------------

# Specify the location containing the software distribution.
VERSION=1.2.8
DOWNLOAD=http://www.phys.uu.nl/~rombouts/pdnsd/releases/pdnsd-$VERSION-par.tar.gz

#-----------------------------------------------------------------------

# Download if necessary.
tar tzf $TMP/$(basename $DOWNLOAD) 1>/dev/null 2>&1 ||
  wget -O- $DOWNLOAD >$TMP/$(basename $DOWNLOAD)

#-----------------------------------------------------------------------

# Save the download for GPL compliance.
cp $TMP/$(basename $DOWNLOAD) $PACKAGE

#-----------------------------------------------------------------------

# Compute the name of the directory into which the software will be unpacked.
DIRECTORY=$TMP/$EXTENSION0-$VERSION

#-----------------------------------------------------------------------

# Unpack the software.
rm -fR $DIRECTORY
tar xzf $TMP/$(basename $DOWNLOAD) -C $TMP

#-----------------------------------------------------------------------

export CFLAGS="-march=i486 -mtune=i686 -Os -pipe"
export CXXFLAGS="-march=i486 -mtune=i686 -Os -pipe"
export LDFLAGS="-Wl,-O1"

#-----------------------------------------------------------------------

# Build the software.
{ cd $DIRECTORY ; ./configure ; make ; 
  touch $TMP/TIMESTAMP ; sudo make install ; }

#-----------------------------------------------------------------------

# Copy the installed files (those newer than $TMP/TIMESTAMP) from 
# /usr/local to $SQUASHFS/usr/local.
( cd / 
  find usr/local -not -type d -newer $TMP/TIMESTAMP |
    grep -v /man/ |
    cpio -pvd $SQUASHFS
)

#-----------------------------------------------------------------------

# Make a default configuration file.
mkdir -p $SQUASHFS/usr/local/etc/
cat >$SQUASHFS/usr/local/etc/$EXTENSION.conf.default <<EOF

# Define global parameters that apply to all sections.
global {
  cache_dir = "/var/cache/pdnsd";  # Location of the cache directory.
  pid_file = "/var/run/pdnsd.pid"; # Location of the PID file.
  run_as = "nobody";               # Execute as this user.
  perm_cache = 1024;               # Size of disk cache in KB.    
  server_ip = 127.0.0.1;           # Listen for requests on 127.0.0.1:53.
  status_ctl = on;                 # Enable pdsnd-ctl.
# paranoid = on;                   # Reduces the chance of cache poisoning,
                                   # which may make pdnsd less efficient.
  query_method = tcp_udp;          # Only make requests using first TCP and
                                   # the UDP. Change to tcp_only to not use UDP.
  min_ttl = 15m;                   # Retain cached entries at least 15 minutes.
  max_ttl = 1w;                    # Retain cached entries for at most one week.
  timeout = 30;                    # Global timeout option in seconds.
  daemon = off;                    # Do not start as a daemon.
}

# Define a set on domain name servers.
server {
  ip =                             # This is the list of DNS servers;
    8.8.8.8 ,                      #   Google
    8.8.4.4 ,                      #   Google
    208.67.222.222 ,               #   OpenDNS
    208.67.222.222                 #   OpenDNS
    ;                              #
  label = "MyServers";             # Name this section.
  randomize_servers = on;          # Randomize the selection of DNS servers.
  uptest = none;                   # Do not send messages to test the 
                                   # availability of servers.  Only the query
                                   # method works for TCP, but not with Google.
  preset = yes;                    # Assume servers are up so uptest=none works.
  timeout = 10;                    # Timeout in seconds for requests.
  proxy_only = on;                 # Accept responses with querying secondary
                                   # servers.
  lean_query = on;                 # Only request the needed DNS records.
  purge_cache = off;               # Keep cached records beyond their time-to-
                                   # live if servers are down.
}

# Get definitions from /etc/hosts.
source {
	owner=localhost;
#	serve_aliases=on;
	file="/etc/hosts";
}

# Define a DNS resource record for localhost.
rr {
  name=localhost;
  reverse=on;
  a=127.0.0.1;
  owner=localhost;
  soa=localhost,root.localhost,42,86400,900,86400,86400;
}
EOF


#-----------------------------------------------------------------------

# Create a script to start and stop pdnsd.
mkdir -p $SQUASHFS/usr/local/etc/init.d
cat >$SQUASHFS/usr/local/etc/init.d/$EXTENSION <<'EOF'
#!/bin/sh

# This is a script to start and stop the pdnsd daemon.

# This script must be executed as root.
[ "$(id -u)" != '0' ] && echo $0 must be executed as root. && exit 1

# The udhcpc default script must be modified so we can control
# /etc/resolv.conf.
DEFAULT_SCRIPT=/usr/share/udhcpc/default.script
DEFAULT_NAMESERVERS="$(dirname $DEFAULT_SCRIPT)/default.nameservers"

# Change the udhcpc script $DEFAULT_SCRIPT so it checks for the file
# $DEFAULT_NAMESERVERS containing a list of name server IP addresses.
if ! grep -q $DEFAULT_NAMESERVERS $DEFAULT_SCRIPT ; then
  sed -i '/RESOLV_CONF=/ i\
# Replace dns, the DHCP-supplied list of name servers, with the content \
# of the file '$DEFAULT_NAMESERVERS' if that file exists. \
[ -r '$DEFAULT_NAMESERVERS' ] && \
  dns="$(cat '$DEFAULT_NAMESERVERS')" \
 \
' $DEFAULT_SCRIPT
fi 

# If $DEFAULT_NAMESERVERS exits and is not "127.0.0.1", 
# save it in $DEFAULT_NAMESERVERS.saved.
save_nameservers () {
  [ -r $DEFAULT_NAMESERVERS ] &&
    [ "$(cat $DEFAULT_NAMESERVERS)" != '127.0.0.1' ] &&
      mv -f $DEFAULT_NAMESERVERS $DEFAULT_NAMESERVERS.saved
}

# If $DEFAULT_NAMESERVERS.saved exists, use is to restore 
# $DEFAULT_NAMESERVERS; otherwise, delete $DEFAULT_NAMESERVERS.
restore_nameservers () {
  if [ -r $DEFAULT_NAMESERVERS.saved ] ; then
    mv -f $DEFAULT_NAMESERVERS.saved $DEFAULT_NAMESERVERS
  else  
    rm -f $DEFAULT_NAMESERVERS
  fi
}

# Create /var/cache/pdnsd, switch the name server to 127.0.0.1, signal
# udhcpc to rebuild /etc/resolv.conf, and start pdnsd using either
# "-d" or the options specified on the command line.
start () {
  if [ ! -d /var/cache/pdnsd ] ; then
    mkdir -p /var/cache/pdnsd
    chown nobody:nogroup /var/cache/pdnsd
  fi
  save_nameservers
  echo 127.0.0.1 >$DEFAULT_NAMESERVERS 
  killall -q -USR1 udhcpc 
  pkill -f '^/usr/local/sbin/pdnsd '
  shift
  /usr/local/sbin/pdnsd ${@:-'-d'}
  return 0
}

# Restore the name servers, signal udhcpc to rebuild /etc/resolv.conf,
# and terminate pdnsd.
stop () {
  restore_nameservers
  killall -q -USR1 udhcpc 
  pkill -f '^/usr/local/sbin/pdnsd '
  return 0
}

case "$1" in
start)   start "$@" ;;
stop)    stop  ;;
restart) stop && start "$@" ;;
*)       echo Usage: $0 '{ start [options] | stop | restart [options] }' ;;
esac
EOF
chmod +x $SQUASHFS/usr/local/etc/init.d/$EXTENSION

#-----------------------------------------------------------------------

# Create an initialization script that creates /var/cache/pdnsd and
# the file /usr/local/etc/$EXTENSION.conf if the user has not supplied 
# a version of that file.
mkdir -p $SQUASHFS/usr/local/tce.installed
cat >$SQUASHFS/usr/local/tce.installed/$EXTENSION <<EOF
#!/bin/sh

# Create the directory to hold the cache.
mkdir -p /var/cache/pdnsd
chown nobody:nogroup /var/cache/pdnsd

# Create /usr/local/etc/$EXTENSION.conf if the user has not supplied it.
[ ! -r /usr/local/etc/$EXTENSION.conf ] &&
  cp /usr/local/etc/$EXTENSION.conf.default /usr/local/etc/$EXTENSION.conf
EOF
sudo chmod +x $SQUASHFS/usr/local/tce.installed/$EXTENSION

#-----------------------------------------------------------------------

# Create $EXTENSION.tcz.info.
cat >$PACKAGE/$EXTENSION.tcz.info <<EOF
Title:          $EXTENSION.tcz
Description:    Proxy DNS server with permanent caching
Version:        $VERSION
Author:         Paul Rombouts
Original-site:  http://www.phys.uu.nl/~rombouts/pdnsd/index.html
Source:         $DOWNLOAD
Copying-policy: GPL v3
Size:           SiZeK
Extension_by:   vitex
Comments:       -----

                From the web page:

                  pdnsd is a proxy DNS server with permanent caching
                  (the cache contents are written to hard disk on
                  exit) that is designed to cope with unreachable or
                  down DNS servers (for example in dial-in
                  networking)....  pdnsd can be used with applications
                  that do DNS lookups, e.g. on startup, and can't be
                  configured to change that behaviour, to prevent the
                  often minute-long hangs (or even crashes) that
                  result from stalled DNS queries.... pdnsd is
                  configurable via a file and supports run-time
                  configuration using the program pdnsd-ctl that comes
                  with pdnsd. This allows you to set the status flags
                  of servers that pdnsd knows (to influence which
                  servers pdnsd will query), and the addition,
                  deletion and invalidation of DNS records in pdnsd's
                  cache.

                The configuration file for pdnsd is

                  /usr/local/etc/pdnsd.conf

                which receives its initial value from

                  /usr/local/etc/pdnsd.conf.default

                That configuraton uses public servers from Google and
                OpenDNS.  Change those servers as needed and add the
                configuration file to you back-up set.  The sample
                configuration file from the pdnsd distribution is

                  /usr/local/etc/pdnsd.conf.sample

                The extension installs the script

                  /usr/local/etc/init.d/pdnsd

                to simplify usage of pdnsd.  Executing (e.g., in
                /opt/bootlocal.sh)

                  sudo /usr/local/etc/init.d/pdnsd start

                will change /etc/resolv.conf to point to pdnsd and use
                the "-d" option to activate pdnsd as a daemon.  Add a
                "-g" and a "-d" option 

                  sudo /usr/local/etc/init.d/pdnsd start -g -d

                to execute pdnsd as a daemon and to record debug
                messages in the file

                  /var/cache/pdnsd/pdnsd.debug

                Use the "-g" option alone to execute in the foreground
                and see the debug messages on the console.  Executing

                  sudo /usr/local/etc/init.d/pdnsd stop

                will terminate pdnsd and signal udhcpc to rewrite
                /etc/resolv.conf with the domain name servers that
                it obtains by querying its up-stream DHCP server.

                The DNS cache is saved to the disk file

                  /var/cache/pdnsd/pdnsd.cache

                when pdnsd terminates.  That file must be backed up if
                you wish to take advantage of pdnsd's persistent
                cache.  To test cache behavior, use

                  sudo pdnsd-ctl empty-cache

                to signal pdnsd to empty its cache. For example,

                  L=tinycorelinux.net ; sudo pdnsd-ctl empty-cache ; \\
                    time nslookup \$L ; time nslookup \$L

                should show the second lookup being almost instantaneous. 

                The default configuration sets 

                  query_method = tcp_udp;

                which forces all up-stream DNS queries to be sent
                first using TCP and then using UDP if TCP fails.  That
                parameter can be changed to

                  query_method = tcp_only;

                or pdnsd can be invoked with the "-mto" option to
                force pdnsd to use TCP but to not fall back to UDP.
                Only using TCP is important for some encrypting
                proxies (e.g., the OpenSSH SOCKS proxy) that do not
                carry UDP traffic, in which case UDP DNS queries are
                not protected by the proxy and might reveal the sites
                that are being visited.

Change-log:     ----------
Current:	2011/10/09 First version
EOF

#-----------------------------------------------------------------------
